@@ -5,7 +5,7 @@ use IRI 0.003;
 use Types::Namespace 0.004 qw( Iri );
 use namespace::autoclean;
 
-our $VERSION = '1.00';
+our $VERSION = '1.05_01';
 
 =head1 NAME
 
@@ -82,7 +82,10 @@ has _uri => (
 sub iri {
 	my ($self, $name) = @_;
 	if (defined($name)) {
-		return IRI->new($self->_uri->as_string . "$name");
+		my $str = $self->_uri->as_string;
+		# XSD is given without hash in XML world, but hash should be added anyway (see issue #14)
+		$str .= '#' if ($str eq 'http://www.w3.org/2001/XMLSchema');
+		return IRI->new($str . "$name");
 	} else {
 		return $self->_uri;
 	}
@@ -92,6 +95,7 @@ sub uri {
 	my ($self, $name) = @_;
 	my $iri = $self->_uri->as_string;
 	if (defined($name)) {
+		$iri .= '#' if ($iri eq 'http://www.w3.org/2001/XMLSchema');
 		return URI->new($iri . "$name");
 	} else {
 		return URI->new($iri);
