@@ -1,6 +1,4 @@
 package URI::NamespaceMap;
-use strict;
-use warnings;
 use Moo 1.006000;
 use Module::Load::Conditional qw[can_load];
 use URI::Namespace;
@@ -10,6 +8,7 @@ use Sub::Quote qw( quote_sub );
 use Try::Tiny;
 use Types::Standard qw(HashRef);
 use Types::Namespace 0.004 qw(Namespace);
+use URI::NamespaceMap::ReservedLocalParts;
 use namespace::autoclean;
 
 =head1 NAME
@@ -18,11 +17,11 @@ URI::NamespaceMap - Class holding a collection of namespaces
 
 =head1 VERSION
 
-Version 0.32
+Version 1.05_01
 
 =cut
 
-our $VERSION = '0.32';
+our $VERSION = '1.05_01';
 
 
 =head1 SYNOPSIS
@@ -35,6 +34,10 @@ our $VERSION = '0.32';
   $map->add_mapping(rdf => 'http://www.w3.org/1999/02/22-rdf-syntax-ns#' );
   $map->list_prefixes;  #  ( 'foaf', 'rdf', 'xsd' )
   $map->foaf; # Returns URI::Namespace object
+  while (my ($prefix, $nsURI) = $map->each_map) {
+	 $node->setNamespace($nsURI->as_string, $prefix); # For use with XML::LibXML
+  }
+
 
 =head1 DESCRIPTION
 
@@ -89,19 +92,32 @@ Returns an array of L<URI::Namespace> objects with all the namespaces.
 
 Returns an array of prefixes.
 
-=cut
+=item C<< each_map >>
 
+Returns an 2-element list where the first element is a prefix and the
+second is the corresponding L<URI::Namespace> object.
+
+=cut
 
 around BUILDARGS => sub {
 	my ($next, $self, @parameters) = @_;
 	if (ref($parameters[0]) eq 'ARRAY') {
 		return { namespace_map => $self->_guess(@{$parameters[0]}) };
 	}
-	return $self->$next(@parameters) if (@parameters > 1);
-	return $self->$next(@parameters) if (exists $parameters[0]->{namespace_map});
-	return { namespace_map => $parameters[0] };
+	if (@parameters == 1 && (! exists $parameters[0]->{namespace_map})) {
+		return { namespace_map => $parameters[0] } 
+	} else { $self->$next(@parameters) }
 };
 
+sub BUILD {
+    my ($self, $args) = @_;
+    my $r = URI::NamespaceMap::ReservedLocalParts->new(disallowed => [qw/uri/]);
+    for my $local_part (keys %{$args->{namespace_map}}) {
+        Carp::croak("$_[1] prohibited as local part")
+            if $r->is_reserved($local_part);
+    }
+}
+
 has namespace_map => (
                       is => "ro",
                       isa => HashRef[Namespace],
@@ -109,11 +125,17 @@ has namespace_map => (
                       default => quote_sub q { {} },
                      );
 
-sub add_mapping     { $_[0]->namespace_map->{$_[1]} = Namespace->assert_coerce($_[2]) }
+sub add_mapping {
+	my $r = URI::NamespaceMap::ReservedLocalParts->new(disallowed => [qw/uri/]);
+	Carp::croak("$_[1] prohibited as local part") if $r->is_reserved($_[1]);
+
+	$_[0]->namespace_map->{$_[1]} = Namespace->assert_coerce($_[2])
+}
 sub remove_mapping  { delete $_[0]->namespace_map->{$_[1]} }
 sub namespace_uri   { $_[0]->namespace_map->{$_[1]} }
 sub list_namespaces { values %{ $_[0]->namespace_map } }
 sub list_prefixes   { keys   %{ $_[0]->namespace_map } }
+sub each_map        { each   %{ $_[0]->namespace_map } }
 
 
 =item C<< guess_and_add ( @string_or_uri ) >>
@@ -132,8 +154,6 @@ sub guess_and_add {
 	}
 }
 
-
-
 =item C<< uri ( $prefixed_name ) >>
 
 Returns a URI for an abbreviated string such as 'foaf:Person'.
@@ -166,7 +186,6 @@ called in list context) for the given URI.
 
 =cut
 
-
 # turn the URI back into a string to mitigate unexpected behaviour
 sub _scrub_uri {
 	my $uri = shift;
@@ -197,36 +216,36 @@ sub _scrub_uri {
 			            "an unblessed %s reference", ref $uri);
 		}
 	}
-	
+
 	return $uri;
 }
 
 sub prefix_for {
 	my ($self, $uri) = @_;
-	
+
 	$uri = _scrub_uri($uri);
-	
+
 	my @candidates;
 	for my $k ($self->list_prefixes) {
 		my $v = $self->namespace_uri($k);
-		
+
 		my $nsuri = $v->as_string;
-		
+
 		# the input should always be longer than the namespace
 		next if length $nsuri > length $uri;
-		
+
 		# candidate namespace must match exactly
 		my $cns = substr($uri, 0, length $nsuri);
 		push @candidates, $k if $cns eq $nsuri;
 	}
-	
+
 	# make sure this behaves correctly when empty
 	return unless @candidates;
-	
+
 	# if this returns more than one prefix, take the
 	# shortest/lexically lowest one.
 	@candidates = sort @candidates;
-	
+
 	return wantarray ? @candidates : $candidates[0];
 }
 
@@ -243,16 +262,16 @@ may be useful for certain serialization tasks.
 
 sub abbreviate {
 	my ($self, $uri) = @_;
-	
+
 	$uri = _scrub_uri($uri);
-	
+
 	my $prefix = $self->prefix_for($uri);
-	
+
 	# XXX is this actually the most desirable behaviour?
 	return unless defined $prefix;
-	
+
 	my $nsuri = _scrub_uri($self->namespace_uri($prefix));
-	
+
 	return sprintf('%s:%s', $prefix, substr($uri, length $nsuri));
 }
 
@@ -272,31 +291,39 @@ sub _guess {
 	my $xmlns = can_load( modules => { 'XML::CommonNS' => 0 } );
 	my $rdfns = can_load( modules => { 'RDF::NS' => 20130802 } );
 	my $rdfpr = can_load( modules => { 'RDF::Prefixes' => 0 } );
-	
+
 	confess 'To resolve an array, you need at least one of RDF::NS::Curated, XML::CommonNS, RDF::NS or RDF::Prefixes' unless ($rnscu || $xmlns || $rdfns || $rdfpr);
 	my %namespaces;
-	
+	my $r = URI::NamespaceMap::ReservedLocalParts->new(disallowed => [qw/uri/]);
+
 	foreach my $entry (@data) {
-		
 		if ($entry =~ m/^[a-z]\w+$/i) {
 			# This is a prefix
 			carp "Cannot resolve '$entry' without RDF::NS::Curated, XML::CommonNS, RDF::NS" unless ($rnscu || $xmlns || $rdfns);
+			my $i = 1;
+			my $prefix = $entry;
+			while ($r->is_reserved($prefix)) {
+				$prefix .= 'x';
+				carp "Cannot resolve '$entry' as tried prefix '$prefix' conflicts with method names." if ($i > 5);
+				$i++;
+			}
+
 			if ($rnscu) {
 				my $ns = RDF::NS::Curated->new;
-				$namespaces{$entry} = $ns->uri($entry);
+				$namespaces{$prefix} = $ns->uri($entry);
 			}
-			if ((! $namespaces{$entry}) && $xmlns) {
+			if ((! $namespaces{$prefix}) && $xmlns) {
 				require XML::CommonNS;
 				XML::CommonNS->import(':all');
 				try {
-					$namespaces{$entry} = XML::CommonNS->uri(uc($entry))->toString;
+					$namespaces{$prefix} = XML::CommonNS->uri(uc($entry))->toString;
 				}; # Then, XML::CommonNS doesn't have the prefix, which is OK, we just continue
 			}
-			if ((! $namespaces{$entry}) && $rdfns) {
+			if ((! $namespaces{$prefix}) && $rdfns) {
 				my $ns = RDF::NS->new;
-				$namespaces{$entry} = $ns->SELECT($entry);
+				$namespaces{$prefix} = $ns->SELECT($entry);
 			}
-			carp "Cannot resolve '$entry'" unless $namespaces{$entry};
+			carp "Cannot resolve '$entry'" unless $namespaces{$prefix};
 		} else {
 			# Lets assume a URI string
 			carp "Cannot resolve '$entry' without RDF::NS::Curated, RDF::NS or RDF::Prefixes" unless ($rnscu || $rdfns || $rdfpr);
@@ -316,6 +343,12 @@ sub _guess {
 			unless ($prefix) {
 				carp "Cannot resolve '$entry'";
 			} else {
+				my $i = 1;
+				while ($r->is_reserved($prefix)) {
+					$prefix .= 'x';
+					carp "Cannot resolve '$entry' as tried prefix '$prefix' conflicts with method names." if ($i > 5);
+					$i++;
+				}
 				$namespaces{$prefix} = $entry;
 			}
 		}
@@ -323,7 +356,6 @@ sub _guess {
 	return \%namespaces;
 }
 
-
 =back
 
 =head1 WARNING
@@ -333,6 +365,8 @@ prefix, because these names are defined as method for every Perl
 object by default. The method names 'new' and 'uri' are also
 forbidden. Names of methods of L<Moose::Object> must also be avoided.
 
+Using them will result in an error.
+
 =head1 AUTHORS
 
 Chris Prather, C<< <chris@prather.org> >>
@@ -343,6 +377,7 @@ Toby Inkster, C<< <tobyink@cpan.org> >>
 =head1 CONTRIBUTORS
 
 Dorian Taylor
+Paul Williams
 
 =head1 BUGS
 
@@ -357,7 +392,7 @@ You can find documentation for this module with the perldoc command.
 
 =head1 COPYRIGHT & LICENSE
 
-Copyright 2012,2013,2014,2015,2016 Gregory Todd Williams, Chris Prather and Kjetil Kjernsmo
+Copyright 2012,2013,2014,2015,2016,2017,2018 Gregory Todd Williams, Chris Prather and Kjetil Kjernsmo
 
 This program is free software; you can redistribute it and/or modify it
 under the same terms as Perl itself.
@@ -367,4 +402,3 @@ under the same terms as Perl itself.
 
 1;
 __END__
-
